package com.margelo.nitro.multipleimagepicker.utils

import android.content.ContentResolver
import android.content.ContentUris
import android.content.Context
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.ImageDecoder
import android.net.Uri
import android.os.Build
import android.provider.DocumentsContract
import android.provider.MediaStore
import android.util.Log
import java.io.IOException
import kotlin.math.sqrt

//java.lang.RuntimeException: Canvas: trying to draw too large(xxx bytes) bitmap.
//该异常由 android.graphics.RecordingCanvas.java 或 android.view.DisplayListCanvas.java （SDK版本差异）的 throwIfCannotDraw(Bitmap bitmap) 抛出。
//阅读源码可知加载的图片内存大小超过 MAX_BITMAP_SIZE，因此控制图片的加载内存即可解决。
const val MAX_BITMAP_SIZE = 64f * 1024 * 1024 // 64 MB

fun Context.getBitmapFromPath(path: String, targetWidth: Int = 0): Bitmap? {
    try {
        val bitmapOptions = BitmapFactory.Options()
        bitmapOptions.inJustDecodeBounds = true
        BitmapFactory.decodeFile(path, bitmapOptions)
        val bitmapWidth = bitmapOptions.outWidth
        val bitmapHeight = bitmapOptions.outHeight
        //bitmapSize = 图片宽度 * 图片高度 * 色彩模式 （ARGB_8888 = 4byte）
        val bitmapSize = bitmapWidth * bitmapHeight * 4
        bitmapOptions.inJustDecodeBounds = false
        if (bitmapSize > MAX_BITMAP_SIZE || targetWidth > 0) {
            val maxWidth = (bitmapWidth * sqrt(MAX_BITMAP_SIZE / bitmapSize)).toInt()
            bitmapOptions.inScaled = true
            bitmapOptions.inDensity = bitmapWidth
            bitmapOptions.inTargetDensity = targetWidth.coerceAtMost(maxWidth)
        }
        return BitmapFactory.decodeFile(path, bitmapOptions)
    } catch (e: Exception) {
        Log.e(this.javaClass.name, e.message.toString())
    }
    return null
}

fun Context.getBitmapFromUri(uri: Uri, targetWidth: Int = 0): Bitmap? {
    if (Build.VERSION.SDK_INT < Build.VERSION_CODES.P) {
        return null
    }
    if (ContentResolver.SCHEME_CONTENT != uri.scheme && ContentResolver.SCHEME_FILE != uri.scheme) {
        return null
    }
    return try {
        val src = ImageDecoder.createSource(contentResolver, uri)
        ImageDecoder.decodeBitmap(src) { decoder, info, _ ->
            val bitmapWidth = info.size.width
            val bitmapHeight = info.size.height
            //bitmapSize = 图片宽度 * 图片高度 * 色彩模式 （ARGB_8888 = 4byte）
            val bitmapSize = bitmapWidth * bitmapHeight * 4
            if (bitmapSize > MAX_BITMAP_SIZE || targetWidth > 0) {
                val maxWidth = bitmapWidth * sqrt(MAX_BITMAP_SIZE / bitmapSize)
                val scale = targetWidth.toFloat().coerceAtMost(maxWidth) / bitmapWidth
                decoder.setTargetSize(
                    (bitmapWidth * scale).toInt(),
                    (bitmapHeight * scale).toInt()
                )
            }
            decoder.allocator = ImageDecoder.ALLOCATOR_SOFTWARE
        }
    } catch (e: IOException) {
        Log.e(this.javaClass.name, e.message.toString())
        null
    }
}

fun Context.getBitmapPathFromUri(uri: Uri): String {
    var imagePath = ""
    if (DocumentsContract.isDocumentUri(this, uri)) {
        val docId = DocumentsContract.getDocumentId(uri)
        if ("com.android.providers.media.documents" == uri.authority) {
            val id = docId.split(":".toRegex()).toTypedArray()[1]
            val contentUri = MediaStore.Images.Media.EXTERNAL_CONTENT_URI
            val selection = MediaStore.Images.Media._ID + "=" + id
            imagePath = contentResolverQueryPath(contentUri, selection)
        } else if ("com.android.providers.downloads.documents" == uri.authority) {
            val uriString = "content://downloads/public_downloads"
            val contentUri = ContentUris.withAppendedId(Uri.parse(uriString), docId.toLong())
            imagePath = contentResolverQueryPath(contentUri)
        }
    } else if ("content".equals(uri.scheme, ignoreCase = true)) {
        imagePath = contentResolverQueryPath(uri)
    } else if ("file".equals(uri.scheme, ignoreCase = true)) {
        imagePath = uri.path.toString()
    }
    return imagePath
}

fun Context.contentResolverQueryPath(uri: Uri, selection: String = ""): String {
    val cursor = contentResolver.query(uri, null, selection, null, null)
    if (cursor != null) {
        if (cursor.moveToFirst()) {
            val index = cursor.getColumnIndex(MediaStore.Images.Media.DATA)
            return cursor.getString(index)
        }
        cursor.close()
    }
    return ""
}